@@ -6,6 +6,8 @@ use strict;
 use Algorithm::KMeans;
 
 my $datafile = "mydatafile1.dat";
+#my $datafile = "features_temp.txt";
+#my $datafile = "fernando.dat";
 
 # Mask:
 
@@ -20,9 +22,12 @@ my $datafile = "mydatafile1.dat";
 # "N011".  The symbolic ID can be in any column --- you just
 # have to place the character `N' at the right place:
 
-my $mask = "N111";
+my $mask = "N111";        # for mydatafile1.dat 
+#my $mask = "N011";
+#my $mask = "N100";
+#my $mask = "N10";          # for fernando.dat and features_temp.txt datafiles
 my $clusterer = Algorithm::KMeans->new( datafile => $datafile,
-                                        mask     => "N111",
+                                        mask     => $mask,
                                         K        => 3,
                                         terminal_output => 1,
 #                                        write_clusters_to_files => 1,
@@ -53,7 +58,9 @@ foreach my $cluster (@$clusters) {
 # visualization must not exceed the number of on bits in the
 # original data mask.
 
-my $visualization_mask = "111";
+my $visualization_mask = "111";    # for mydatafile1.dat 
+#my $visualization_mask = "11";
+#my $visualization_mask = "1";       # for fernando.dat and features_temp.txt
 
 $clusterer->visualize_clusters($visualization_mask);
 
